@@ -96,7 +96,7 @@
 
 int jpc_ft_analyze(jpc_fix_t *a, int xstart, int ystart, int width, int height,
   int stride);
-int jpc_ft_synthesize(int *a, int xstart, int ystart, int width, int height,
+int jpc_ft_synthesize(jpc_fix_t *a, int xstart, int ystart, int width, int height,
   int stride);
 
 int jpc_ns_analyze(jpc_fix_t *a, int xstart, int ystart, int width, int height,
@@ -306,11 +306,7 @@ void jpc_qmfb_split_row(jpc_fix_t *a, int numcols, int parity)
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numcols, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t splitbuf[QMFB_SPLITBUFSIZE];
-#else
-	jpc_fix_t splitbuf[bufsize];
-#endif
 	jpc_fix_t *buf = splitbuf;
 	register jpc_fix_t *srcptr;
 	register jpc_fix_t *dstptr;
@@ -318,7 +314,6 @@ void jpc_qmfb_split_row(jpc_fix_t *a, int numcols, int parity)
 	register int m;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Get a buffer. */
 	if (bufsize > QMFB_SPLITBUFSIZE) {
 		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
@@ -326,11 +321,12 @@ void jpc_qmfb_split_row(jpc_fix_t *a, int numcols, int parity)
 			abort();
 		}
 	}
-#endif
 
 	if (numcols >= 2) {
 		hstartcol = (numcols + 1 - parity) >> 1;
-		m = (parity) ? hstartcol : (numcols - hstartcol);
+		// ORIGINAL (WRONG): m = (parity) ? hstartcol : (numcols - hstartcol);
+		m = numcols - hstartcol;
+
 		/* Save the samples destined for the highpass channel. */
 		n = m;
 		dstptr = buf;
@@ -360,12 +356,10 @@ void jpc_qmfb_split_row(jpc_fix_t *a, int numcols, int parity)
 		}
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the split buffer was allocated on the heap, free this memory. */
 	if (buf != splitbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -374,19 +368,14 @@ void jpc_qmfb_split_col(jpc_fix_t *a, int numrows, int stride,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t splitbuf[QMFB_SPLITBUFSIZE];
-#else
-	jpc_fix_t splitbuf[bufsize];
-#endif
 	jpc_fix_t *buf = splitbuf;
 	register jpc_fix_t *srcptr;
 	register jpc_fix_t *dstptr;
 	register int n;
 	register int m;
-	int hstartcol;
+	int hstartrow;
 
-#if !defined(HAVE_VLA)
 	/* Get a buffer. */
 	if (bufsize > QMFB_SPLITBUFSIZE) {
 		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
@@ -394,11 +383,12 @@ void jpc_qmfb_split_col(jpc_fix_t *a, int numrows, int stride,
 			abort();
 		}
 	}
-#endif
 
 	if (numrows >= 2) {
-		hstartcol = (numrows + 1 - parity) >> 1;
-		m = (parity) ? hstartcol : (numrows - hstartcol);
+		hstartrow = (numrows + 1 - parity) >> 1;
+		// ORIGINAL (WRONG): m = (parity) ? hstartrow : (numrows - hstartrow);
+		m = numrows - hstartrow;
+
 		/* Save the samples destined for the highpass channel. */
 		n = m;
 		dstptr = buf;
@@ -418,7 +408,7 @@ void jpc_qmfb_split_col(jpc_fix_t *a, int numrows, int stride,
 			srcptr += stride << 1;
 		}
 		/* Copy the saved samples into the highpass channel. */
-		dstptr = &a[hstartcol * stride];
+		dstptr = &a[hstartrow * stride];
 		srcptr = buf;
 		n = m;
 		while (n-- > 0) {
@@ -428,12 +418,10 @@ void jpc_qmfb_split_col(jpc_fix_t *a, int numrows, int stride,
 		}
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the split buffer was allocated on the heap, free this memory. */
 	if (buf != splitbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -442,11 +430,7 @@ void jpc_qmfb_split_colgrp(jpc_fix_t *a, int numrows, int stride,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t splitbuf[QMFB_SPLITBUFSIZE * JPC_QMFB_COLGRPSIZE];
-#else
-	jpc_fix_t splitbuf[bufsize * JPC_QMFB_COLGRPSIZE];
-#endif
 	jpc_fix_t *buf = splitbuf;
 	jpc_fix_t *srcptr;
 	jpc_fix_t *dstptr;
@@ -455,21 +439,22 @@ void jpc_qmfb_split_colgrp(jpc_fix_t *a, int numrows, int stride,
 	register int n;
 	register int i;
 	int m;
-	int hstartcol;
+	int hstartrow;
 
-#if !defined(HAVE_VLA)
 	/* Get a buffer. */
 	if (bufsize > QMFB_SPLITBUFSIZE) {
-		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
+		if (!(buf = jas_alloc3(bufsize, JPC_QMFB_COLGRPSIZE,
+		  sizeof(jpc_fix_t)))) {
 			/* We have no choice but to commit suicide in this case. */
 			abort();
 		}
 	}
-#endif
 
 	if (numrows >= 2) {
-		hstartcol = (numrows + 1 - parity) >> 1;
-		m = (parity) ? hstartcol : (numrows - hstartcol);
+		hstartrow = (numrows + 1 - parity) >> 1;
+		// ORIGINAL (WRONG): m = (parity) ? hstartrow : (numrows - hstartrow);
+		m = numrows - hstartrow;
+
 		/* Save the samples destined for the highpass channel. */
 		n = m;
 		dstptr = buf;
@@ -501,7 +486,7 @@ void jpc_qmfb_split_colgrp(jpc_fix_t *a, int numrows, int stride,
 			srcptr += stride << 1;
 		}
 		/* Copy the saved samples into the highpass channel. */
-		dstptr = &a[hstartcol * stride];
+		dstptr = &a[hstartrow * stride];
 		srcptr = buf;
 		n = m;
 		while (n-- > 0) {
@@ -517,12 +502,10 @@ void jpc_qmfb_split_colgrp(jpc_fix_t *a, int numrows, int stride,
 		}
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the split buffer was allocated on the heap, free this memory. */
 	if (buf != splitbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -531,11 +514,7 @@ void jpc_qmfb_split_colres(jpc_fix_t *a, int numrows, int numcols,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t splitbuf[QMFB_SPLITBUFSIZE * JPC_QMFB_COLGRPSIZE];
-#else
-	jpc_fix_t splitbuf[bufsize * numcols];
-#endif
 	jpc_fix_t *buf = splitbuf;
 	jpc_fix_t *srcptr;
 	jpc_fix_t *dstptr;
@@ -546,19 +525,19 @@ void jpc_qmfb_split_colres(jpc_fix_t *a, int numrows, int numcols,
 	int m;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Get a buffer. */
 	if (bufsize > QMFB_SPLITBUFSIZE) {
-		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
+		if (!(buf = jas_alloc3(bufsize, numcols, sizeof(jpc_fix_t)))) {
 			/* We have no choice but to commit suicide in this case. */
 			abort();
 		}
 	}
-#endif
 
 	if (numrows >= 2) {
 		hstartcol = (numrows + 1 - parity) >> 1;
-		m = (parity) ? hstartcol : (numrows - hstartcol);
+		// ORIGINAL (WRONG): m = (parity) ? hstartcol : (numrows - hstartcol);
+		m = numrows - hstartcol;
+
 		/* Save the samples destined for the highpass channel. */
 		n = m;
 		dstptr = buf;
@@ -606,12 +585,10 @@ void jpc_qmfb_split_colres(jpc_fix_t *a, int numrows, int numcols,
 		}
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the split buffer was allocated on the heap, free this memory. */
 	if (buf != splitbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -619,18 +596,13 @@ void jpc_qmfb_join_row(jpc_fix_t *a, int numcols, int parity)
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numcols, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t joinbuf[QMFB_JOINBUFSIZE];
-#else
-	jpc_fix_t joinbuf[bufsize];
-#endif
 	jpc_fix_t *buf = joinbuf;
 	register jpc_fix_t *srcptr;
 	register jpc_fix_t *dstptr;
 	register int n;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Allocate memory for the join buffer from the heap. */
 	if (bufsize > QMFB_JOINBUFSIZE) {
 		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
@@ -638,7 +610,6 @@ void jpc_qmfb_join_row(jpc_fix_t *a, int numcols, int parity)
 			abort();
 		}
 	}
-#endif
 
 	hstartcol = (numcols + 1 - parity) >> 1;
 
@@ -670,12 +641,10 @@ void jpc_qmfb_join_row(jpc_fix_t *a, int numcols, int parity)
 		++srcptr;
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the join buffer was allocated on the heap, free this memory. */
 	if (buf != joinbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -684,18 +653,13 @@ void jpc_qmfb_join_col(jpc_fix_t *a, int numrows, int stride,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t joinbuf[QMFB_JOINBUFSIZE];
-#else
-	jpc_fix_t joinbuf[bufsize];
-#endif
 	jpc_fix_t *buf = joinbuf;
 	register jpc_fix_t *srcptr;
 	register jpc_fix_t *dstptr;
 	register int n;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Allocate memory for the join buffer from the heap. */
 	if (bufsize > QMFB_JOINBUFSIZE) {
 		if (!(buf = jas_alloc2(bufsize, sizeof(jpc_fix_t)))) {
@@ -703,7 +667,6 @@ void jpc_qmfb_join_col(jpc_fix_t *a, int numrows, int stride,
 			abort();
 		}
 	}
-#endif
 
 	hstartcol = (numrows + 1 - parity) >> 1;
 
@@ -735,12 +698,10 @@ void jpc_qmfb_join_col(jpc_fix_t *a, int numrows, int stride,
 		++srcptr;
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the join buffer was allocated on the heap, free this memory. */
 	if (buf != joinbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -749,11 +710,7 @@ void jpc_qmfb_join_colgrp(jpc_fix_t *a, int numrows, int stride,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t joinbuf[QMFB_JOINBUFSIZE * JPC_QMFB_COLGRPSIZE];
-#else
-	jpc_fix_t joinbuf[bufsize * JPC_QMFB_COLGRPSIZE];
-#endif
 	jpc_fix_t *buf = joinbuf;
 	jpc_fix_t *srcptr;
 	jpc_fix_t *dstptr;
@@ -763,15 +720,14 @@ void jpc_qmfb_join_colgrp(jpc_fix_t *a, int numrows, int stride,
 	register int i;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Allocate memory for the join buffer from the heap. */
 	if (bufsize > QMFB_JOINBUFSIZE) {
-		if (!(buf = jas_alloc2(bufsize, JPC_QMFB_COLGRPSIZE * sizeof(jpc_fix_t)))) {
+		if (!(buf = jas_alloc3(bufsize, JPC_QMFB_COLGRPSIZE,
+		  sizeof(jpc_fix_t)))) {
 			/* We have no choice but to commit suicide. */
 			abort();
 		}
 	}
-#endif
 
 	hstartcol = (numrows + 1 - parity) >> 1;
 
@@ -821,12 +777,10 @@ void jpc_qmfb_join_colgrp(jpc_fix_t *a, int numrows, int stride,
 		srcptr += JPC_QMFB_COLGRPSIZE;
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the join buffer was allocated on the heap, free this memory. */
 	if (buf != joinbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -835,11 +789,7 @@ void jpc_qmfb_join_colres(jpc_fix_t *a, int numrows, int numcols,
 {
 
 	int bufsize = JPC_CEILDIVPOW2(numrows, 1);
-#if !defined(HAVE_VLA)
 	jpc_fix_t joinbuf[QMFB_JOINBUFSIZE * JPC_QMFB_COLGRPSIZE];
-#else
-	jpc_fix_t joinbuf[bufsize * numcols];
-#endif
 	jpc_fix_t *buf = joinbuf;
 	jpc_fix_t *srcptr;
 	jpc_fix_t *dstptr;
@@ -849,7 +799,6 @@ void jpc_qmfb_join_colres(jpc_fix_t *a, int numrows, int numcols,
 	register int i;
 	int hstartcol;
 
-#if !defined(HAVE_VLA)
 	/* Allocate memory for the join buffer from the heap. */
 	if (bufsize > QMFB_JOINBUFSIZE) {
 		if (!(buf = jas_alloc3(bufsize, numcols, sizeof(jpc_fix_t)))) {
@@ -857,7 +806,6 @@ void jpc_qmfb_join_colres(jpc_fix_t *a, int numrows, int numcols,
 			abort();
 		}
 	}
-#endif
 
 	hstartcol = (numrows + 1 - parity) >> 1;
 
@@ -907,12 +855,10 @@ void jpc_qmfb_join_colres(jpc_fix_t *a, int numrows, int numcols,
 		srcptr += numcols;
 	}
 
-#if !defined(HAVE_VLA)
 	/* If the join buffer was allocated on the heap, free this memory. */
 	if (buf != joinbuf) {
 		jas_free(buf);
 	}
-#endif
 
 }
 
@@ -941,7 +887,8 @@ void jpc_ft_fwdlift_row(jpc_fix_t *a, int numcols, int parity)
 		}
 		n = numcols - llen - parity - (parity == (numcols & 1));
 		while (n-- > 0) {
-			hptr[0] -= (lptr[0] + lptr[1]) >> 1;
+			//hptr[0] -= (lptr[0] + lptr[1]) >> 1;
+			hptr[0] -= jpc_fix_asr(lptr[0] + lptr[1], 1);
 			++hptr;
 			++lptr;
 		}
@@ -953,24 +900,28 @@ void jpc_ft_fwdlift_row(jpc_fix_t *a, int numcols, int parity)
 		lptr = &a[0];
 		hptr = &a[llen];
 		if (!parity) {
-			lptr[0] += (hptr[0] + 1) >> 1;
+			//lptr[0] += (hptr[0] + 1) >> 1;
+			lptr[0] += jpc_fix_asr(hptr[0] + 1, 1);
 			++lptr;
 		}
 		n = llen - (!parity) - (parity != (numcols & 1));
 		while (n-- > 0) {
-			lptr[0] += (hptr[0] + hptr[1] + 2) >> 2;
+			//lptr[0] += (hptr[0] + hptr[1] + 2) >> 2;
+			lptr[0] += jpc_fix_asr(hptr[0] + hptr[1] + 2, 2);
 			++lptr;
 			++hptr;
 		}
 		if (parity != (numcols & 1)) {
-			lptr[0] += (hptr[0] + 1) >> 1;
+			//lptr[0] += (hptr[0] + 1) >> 1;
+			lptr[0] += jpc_fix_asr(hptr[0] + 1, 1);
 		}
 
 	} else {
 
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] <<= 1;
+			//lptr[0] <<= 1;
+			lptr[0] = jpc_fix_asl(lptr[0], 1);
 		}
 
 	}
@@ -1003,7 +954,8 @@ void jpc_ft_fwdlift_col(jpc_fix_t *a, int numrows, int stride, int parity)
 		}
 		n = numrows - llen - parity - (parity == (numrows & 1));
 		while (n-- > 0) {
-			hptr[0] -= (lptr[0] + lptr[stride]) >> 1;
+			//hptr[0] -= (lptr[0] + lptr[stride]) >> 1;
+			hptr[0] -= jpc_fix_asr(lptr[0] + lptr[stride], 1);
 			hptr += stride;
 			lptr += stride;
 		}
@@ -1015,24 +967,28 @@ void jpc_ft_fwdlift_col(jpc_fix_t *a, int numrows, int stride, int parity)
 		lptr = &a[0];
 		hptr = &a[llen * stride];
 		if (!parity) {
-			lptr[0] += (hptr[0] + 1) >> 1;
+			//lptr[0] += (hptr[0] + 1) >> 1;
+			lptr[0] += jpc_fix_asr(hptr[0] + 1, 1);
 			lptr += stride;
 		}
 		n = llen - (!parity) - (parity != (numrows & 1));
 		while (n-- > 0) {
-			lptr[0] += (hptr[0] + hptr[stride] + 2) >> 2;
+			//lptr[0] += (hptr[0] + hptr[stride] + 2) >> 2;
+			lptr[0] += jpc_fix_asr(hptr[0] + hptr[stride] + 2, 2);
 			lptr += stride;
 			hptr += stride;
 		}
 		if (parity != (numrows & 1)) {
-			lptr[0] += (hptr[0] + 1) >> 1;
+			//lptr[0] += (hptr[0] + 1) >> 1;
+			lptr[0] += jpc_fix_asr(hptr[0] + 1, 1);
 		}
 
 	} else {
 
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] <<= 1;
+			//lptr[0] <<= 1;
+			lptr[0] = jpc_fix_asl(lptr[0], 1);
 		}
 
 	}
@@ -1072,7 +1028,8 @@ void jpc_ft_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				hptr2[0] -= (lptr2[0] + lptr2[stride]) >> 1;
+				//hptr2[0] -= (lptr2[0] + lptr2[stride]) >> 1;
+				hptr2[0] -= jpc_fix_asr(lptr2[0] + lptr2[stride], 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1096,7 +1053,8 @@ void jpc_ft_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] += (hptr2[0] + 1) >> 1;
+				//lptr2[0] += (hptr2[0] + 1) >> 1;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1107,7 +1065,8 @@ void jpc_ft_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] += (hptr2[0] + hptr2[stride] + 2) >> 2;
+				//lptr2[0] += (hptr2[0] + hptr2[stride] + 2) >> 2;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + hptr2[stride] + 2, 2);
 				++lptr2;
 				++hptr2;
 			}
@@ -1118,7 +1077,8 @@ void jpc_ft_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] += (hptr2[0] + 1) >> 1;
+				//lptr2[0] += (hptr2[0] + 1) >> 1;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1129,7 +1089,8 @@ void jpc_ft_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] <<= 1;
+				//lptr2[0] <<= 1;
+				lptr2[0] = jpc_fix_asl(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -1172,7 +1133,8 @@ void jpc_ft_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				hptr2[0] -= (lptr2[0] + lptr2[stride]) >> 1;
+				//hptr2[0] -= (lptr2[0] + lptr2[stride]) >> 1;
+				hptr2[0] -= jpc_fix_asr(lptr2[0] + lptr2[stride], 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1196,7 +1158,8 @@ void jpc_ft_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] += (hptr2[0] + 1) >> 1;
+				//lptr2[0] += (hptr2[0] + 1) >> 1;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1207,7 +1170,8 @@ void jpc_ft_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] += (hptr2[0] + hptr2[stride] + 2) >> 2;
+				//lptr2[0] += (hptr2[0] + hptr2[stride] + 2) >> 2;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + hptr2[stride] + 2, 2);
 				++lptr2;
 				++hptr2;
 			}
@@ -1218,7 +1182,8 @@ void jpc_ft_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] += (hptr2[0] + 1) >> 1;
+				//lptr2[0] += (hptr2[0] + 1) >> 1;
+				lptr2[0] += jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1229,7 +1194,8 @@ void jpc_ft_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] <<= 1;
+				//lptr2[0] <<= 1;
+				lptr2[0] = jpc_fix_asl(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -1254,17 +1220,20 @@ void jpc_ft_invlift_row(jpc_fix_t *a, int numcols, int parity)
 		lptr = &a[0];
 		hptr = &a[llen];
 		if (!parity) {
-			lptr[0] -= (hptr[0] + 1) >> 1;
+			//lptr[0] -= (hptr[0] + 1) >> 1;
+			lptr[0] -= jpc_fix_asr(hptr[0] + 1, 1);
 			++lptr;
 		}
 		n = llen - (!parity) - (parity != (numcols & 1));
 		while (n-- > 0) {
-			lptr[0] -= (hptr[0] + hptr[1] + 2) >> 2;
+			//lptr[0] -= (hptr[0] + hptr[1] + 2) >> 2;
+			lptr[0] -= jpc_fix_asr(hptr[0] + hptr[1] + 2, 2);
 			++lptr;
 			++hptr;
 		}
 		if (parity != (numcols & 1)) {
-			lptr[0] -= (hptr[0] + 1) >> 1;
+			//lptr[0] -= (hptr[0] + 1) >> 1;
+			lptr[0] -= jpc_fix_asr(hptr[0] + 1, 1);
 		}
 
 		/* Apply the second lifting step. */
@@ -1276,7 +1245,8 @@ void jpc_ft_invlift_row(jpc_fix_t *a, int numcols, int parity)
 		}
 		n = numcols - llen - parity - (parity == (numcols & 1));
 		while (n-- > 0) {
-			hptr[0] += (lptr[0] + lptr[1]) >> 1;
+			//hptr[0] += (lptr[0] + lptr[1]) >> 1;
+			hptr[0] += jpc_fix_asr(lptr[0] + lptr[1], 1);
 			++hptr;
 			++lptr;
 		}
@@ -1288,7 +1258,8 @@ void jpc_ft_invlift_row(jpc_fix_t *a, int numcols, int parity)
 
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] >>= 1;
+			//lptr[0] >>= 1;
+			lptr[0] = jpc_fix_asr(lptr[0], 1);
 		}
 
 	}
@@ -1316,17 +1287,20 @@ void jpc_ft_invlift_col(jpc_fix_t *a, int numrows, int stride, int parity)
 		lptr = &a[0];
 		hptr = &a[llen * stride];
 		if (!parity) {
-			lptr[0] -= (hptr[0] + 1) >> 1;
+			//lptr[0] -= (hptr[0] + 1) >> 1;
+			lptr[0] -= jpc_fix_asr(hptr[0] + 1, 1);
 			lptr += stride;
 		}
 		n = llen - (!parity) - (parity != (numrows & 1));
 		while (n-- > 0) {
-			lptr[0] -= (hptr[0] + hptr[stride] + 2) >> 2;
+			//lptr[0] -= (hptr[0] + hptr[stride] + 2) >> 2;
+			lptr[0] -= jpc_fix_asr(hptr[0] + hptr[stride] + 2, 2);
 			lptr += stride;
 			hptr += stride;
 		}
 		if (parity != (numrows & 1)) {
-			lptr[0] -= (hptr[0] + 1) >> 1;
+			//lptr[0] -= (hptr[0] + 1) >> 1;
+			lptr[0] -= jpc_fix_asr(hptr[0] + 1, 1);
 		}
 
 		/* Apply the second lifting step. */
@@ -1338,7 +1312,8 @@ void jpc_ft_invlift_col(jpc_fix_t *a, int numrows, int stride, int parity)
 		}
 		n = numrows - llen - parity - (parity == (numrows & 1));
 		while (n-- > 0) {
-			hptr[0] += (lptr[0] + lptr[stride]) >> 1;
+			//hptr[0] += (lptr[0] + lptr[stride]) >> 1;
+			hptr[0] += jpc_fix_asr(lptr[0] + lptr[stride], 1);
 			hptr += stride;
 			lptr += stride;
 		}
@@ -1350,7 +1325,8 @@ void jpc_ft_invlift_col(jpc_fix_t *a, int numrows, int stride, int parity)
 
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] >>= 1;
+			//lptr[0] >>= 1;
+			lptr[0] = jpc_fix_asr(lptr[0], 1);
 		}
 
 	}
@@ -1379,7 +1355,8 @@ void jpc_ft_invlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] -= (hptr2[0] + 1) >> 1;
+				//lptr2[0] -= (hptr2[0] + 1) >> 1;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1390,7 +1367,8 @@ void jpc_ft_invlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] -= (hptr2[0] + hptr2[stride] + 2) >> 2;
+				//lptr2[0] -= (hptr2[0] + hptr2[stride] + 2) >> 2;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + hptr2[stride] + 2, 2);
 				++lptr2;
 				++hptr2;
 			}
@@ -1401,7 +1379,8 @@ void jpc_ft_invlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] -= (hptr2[0] + 1) >> 1;
+				//lptr2[0] -= (hptr2[0] + 1) >> 1;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1425,7 +1404,8 @@ void jpc_ft_invlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				hptr2[0] += (lptr2[0] + lptr2[stride]) >> 1;
+				//hptr2[0] += (lptr2[0] + lptr2[stride]) >> 1;
+				hptr2[0] += jpc_fix_asr(lptr2[0] + lptr2[stride], 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1447,7 +1427,8 @@ void jpc_ft_invlift_colgrp(jpc_fix_t *a, int numrows, int stride, int parity)
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] >>= 1;
+				//lptr2[0] >>= 1;
+				lptr2[0] = jpc_fix_asr(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -1479,7 +1460,8 @@ void jpc_ft_invlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] -= (hptr2[0] + 1) >> 1;
+				//lptr2[0] -= (hptr2[0] + 1) >> 1;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1490,7 +1472,8 @@ void jpc_ft_invlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] -= (hptr2[0] + hptr2[stride] + 2) >> 2;
+				//lptr2[0] -= (hptr2[0] + hptr2[stride] + 2) >> 2;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + hptr2[stride] + 2, 2);
 				++lptr2;
 				++hptr2;
 			}
@@ -1501,7 +1484,8 @@ void jpc_ft_invlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] -= (hptr2[0] + 1) >> 1;
+				//lptr2[0] -= (hptr2[0] + 1) >> 1;
+				lptr2[0] -= jpc_fix_asr(hptr2[0] + 1, 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1525,7 +1509,8 @@ void jpc_ft_invlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 			lptr2 = lptr;
 			hptr2 = hptr;
 			for (i = 0; i < numcols; ++i) {
-				hptr2[0] += (lptr2[0] + lptr2[stride]) >> 1;
+				//hptr2[0] += (lptr2[0] + lptr2[stride]) >> 1;
+				hptr2[0] += jpc_fix_asr(lptr2[0] + lptr2[stride], 1);
 				++lptr2;
 				++hptr2;
 			}
@@ -1547,7 +1532,8 @@ void jpc_ft_invlift_colres(jpc_fix_t *a, int numrows, int numcols, int stride,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] >>= 1;
+				//lptr2[0] >>= 1;
+				lptr2[0] = jpc_fix_asr(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -1592,7 +1578,7 @@ int jpc_ft_analyze(jpc_fix_t *a, int xstart, int ystart, int width, int height,
 
 }
 
-int jpc_ft_synthesize(int *a, int xstart, int ystart, int width, int height,
+int jpc_ft_synthesize(jpc_fix_t *a, int xstart, int ystart, int width, int height,
   int stride)
 {
 	int numrows = height;
@@ -1753,7 +1739,8 @@ void jpc_ns_fwdlift_row(jpc_fix_t *a, int numcols, int parity)
 #if defined(WT_LENONE)
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] <<= 1;
+			//lptr[0] <<= 1;
+			lptr[0] = jpc_fix_asl(lptr[0], 1);
 		}
 #endif
 
@@ -1959,7 +1946,8 @@ void jpc_ns_fwdlift_colgrp(jpc_fix_t *a, int numrows, int stride,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] <<= 1;
+				//lptr2[0] <<= 1;
+				lptr2[0] = jpc_fix_asl(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -2167,7 +2155,8 @@ void jpc_ns_fwdlift_colres(jpc_fix_t *a, int numrows, int numcols,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] <<= 1;
+				//lptr2[0] <<= 1;
+				lptr2[0] = jpc_fix_asl(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -2345,7 +2334,8 @@ void jpc_ns_fwdlift_col(jpc_fix_t *a, int numrows, int stride,
 #if defined(WT_LENONE)
 		if (parity) {
 			lptr2 = &a[0];
-			lptr2[0] <<= 1;
+			//lptr2[0] <<= 1;
+			lptr2[0] = jpc_fix_asl(lptr2[0], 1);
 			++lptr2;
 		}
 #endif
@@ -2467,7 +2457,8 @@ void jpc_ns_invlift_row(jpc_fix_t *a, int numcols, int parity)
 #if defined(WT_LENONE)
 		if (parity) {
 			lptr = &a[0];
-			lptr[0] >>= 1;
+			//lptr[0] >>= 1;
+			lptr[0] = jpc_fix_asr(lptr[0], 1);
 		}
 #endif
 
@@ -2673,7 +2664,8 @@ void jpc_ns_invlift_colgrp(jpc_fix_t *a, int numrows, int stride,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < JPC_QMFB_COLGRPSIZE; ++i) {
-				lptr2[0] >>= 1;
+				//lptr2[0] >>= 1;
+				lptr2[0] = jpc_fix_asr(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -2881,7 +2873,8 @@ void jpc_ns_invlift_colres(jpc_fix_t *a, int numrows, int numcols,
 		if (parity) {
 			lptr2 = &a[0];
 			for (i = 0; i < numcols; ++i) {
-				lptr2[0] >>= 1;
+				//lptr2[0] >>= 1;
+				lptr2[0] = jpc_fix_asr(lptr2[0], 1);
 				++lptr2;
 			}
 		}
@@ -3059,7 +3052,8 @@ void jpc_ns_invlift_col(jpc_fix_t *a, int numrows, int stride,
 #if defined(WT_LENONE)
 		if (parity) {
 			lptr2 = &a[0];
-			lptr2[0] >>= 1;
+			//lptr2[0] >>= 1;
+			lptr2[0] = jpc_fix_asr(lptr2[0], 1);
 			++lptr2;
 		}
 #endif
